# -*- coding: utf-8 -*-
from uamobile.factory.base import AbstractUserAgentFactory
from uamobile.ezweb import EZwebUserAgent
from uamobile.parser import EZwebUserAgentParser

class EZwebUserAgentFactory(AbstractUserAgentFactory):
    device_class = EZwebUserAgent
    parser       = EZwebUserAgentParser()




# -*- coding: utf-8 -*-
from uamobile.base import UserAgent, Display
from uamobile.docomodisplaymap import DISPLAYMAP_DOCOMO

class DoCoMoUserAgent(UserAgent):
    """
    NTT DoCoMo implementation
    see also http://www.nttdocomo.co.jp/service/imode/make/content/spec/useragent/index.html

    property "cache" returns cache size as kilobytes unit.

    property "status" means:
    TB | Browsers
    TC | Browsers with image off (only Available in HTTP 5.0)
    TD | Fetching JAR
    TJ | i-Appli
    """
    carrier = 'DoCoMo'
    short_carrier = 'D'

    def __init__(self, *args, **kwds):
        super(DoCoMoUserAgent, self).__init__(*args, **kwds)
        self.ser = None
        self.icc = None
        self.status = None
        self.s = None
        self.c = 5
        self.series = None
        self.vendor = None
        self.html_version = None

    def strip_serialnumber(self):
        """
        strip Device ID(Hardware ID) and FOMA card ID.
        """
        if not self.serialnumber and not self.card_id:
            return super(DoCoMoUserAgent, self).strip_serialnumber()

        # User-Agent contains device ID or FOMA card ID
        ua = self.useragent
        if self.is_foma():
            # TODO
            # should we simply use regular expression?
            if self.serialnumber:
                ua = ua.replace('ser%s' % self.serialnumber, '')

            if self.card_id:
                ua = ua.replace('icc%s' % self.card_id, '')

            return ua.replace(';;', ';').replace(';)', ')').replace('(;', '(')
        else:
            return ua.replace('/ser%s' % self.serialnumber, '')

    def is_docomo(self):
        return True

    def get_cache_size(self):
        return self.c
    cache_size = property(get_cache_size)

    def get_bandwidth(self):
        return self.s
    bandwidth = property(get_bandwidth)

    def get_card_id(self):
        return self.icc
    card_id = property(get_card_id)

    def get_serialnumber(self):
        return self.ser
    serialnumber = property(get_serialnumber)

    def is_gps(self):
        return self.model in ('F661i', 'F505iGPS')

    def is_foma(self):
        return self.version == '2.0'

    def supports_cookie(self):
        """
        Return true if the device supports Cookie.
        for cookie support of i-mode browsers, see
        http://www.nttdocomo.co.jp/info/news_release/page/090519_00.html#p03
        """
        return (self.cache_size == 500)

    def get_guid(self):
        """
        Get iMode ID(guid). For iMode ID, see
        http://www.nttdocomo.co.jp/service/imode/make/content/ip/index.html#imodeid
        """
        try:
            return self.environ['HTTP_X_DCMGUID']
        except KeyError:
            return None
    guid = property(get_guid)

    def make_display(self):
        """
        create a new Display object.
        """
        try:
            params = DISPLAYMAP_DOCOMO[self.model]
        except KeyError:
            params = {}

        if self.display_bytes:
            try:
                params['width_bytes'], params['height_bytes'] = self.display_bytes
            except ValueError:
                pass

        return Display(**params)



# -*- coding: utf-8 -*-
from uamobile.base import UserAgent, Display

class NonMobileUserAgent(UserAgent):
    name = 'NonMobile'
    carrier = 'NonMobile'
    short_carrier = 'N'
    serialnumber = None

    def get_name(self):
        return 'NonMobile'

    def is_nonmobile(self):
        return True

    def supports_cookie(self):
        return True

    def make_display(self):
        """
        create a new Display object.
        """
        return Display()



# -*- coding: utf-8 -*-
from uamobile.cidr import IP, get_ip
from uamobile.factory.docomo import *
from uamobile.factory.ezweb import *
from uamobile.factory.softbank import *
from uamobile.factory.willcom import *

def normalize_carrier_name(carrier):
    return carrier.lower()

class Context(object):
    def __init__(self, proxy_ips=None,
                 extra_docomo_ips=None, extra_ezweb_ips=None, extra_softbank_ips=None, extra_willcom_ips=None,
                 extra_crawler_ips=None,
                 docomo_factory=None, ezweb_factory=None, softbank_factory=None, willcom_factory=None):
        # extra IPs
        self._extra_ips = {}
        for carrier, ips in (('docomo', extra_docomo_ips),
                             ('ezweb', extra_ezweb_ips),
                             ('softbank', extra_softbank_ips),
                             ('willcom', extra_willcom_ips),
                             ('crawler', extra_crawler_ips),
                             ('nonmobile', None),
                             ):
            extra = self._extra_ips.setdefault(carrier, [])
            if not ips:
                # extra_ip is None or empty sequence
                continue

            if not isinstance(ips, (list, tuple)):
                ips = [ips]
            for addr in ips:
                self.add_ip(carrier, addr)

        # factary classes
        self.docomo_factory = docomo_factory or DoCoMoUserAgentFactory
        self.ezweb_factory = ezweb_factory or EZwebUserAgentFactory
        self.softbank_factory = softbank_factory or SoftBankUserAgentFactory
        self.willcom_factory = willcom_factory or WillcomUserAgentFactory

        if proxy_ips:
            if not isinstance(proxy_ips, (list, tuple)):
                proxy_ips = [proxy_ips]
        self._proxy_ips = proxy_ips

    def get_proxy_ips(self):
        hosts = []
        if self._proxy_ips:
            for addr in self._proxy_ips:
                try:
                    hosts.append(IP(addr))
                except ValueError:
                    raise ValueError('"%s" is not valid reverse proxy address' % addr)

        return hosts

    def get_ip(self, carrier):
        carrier = normalize_carrier_name(carrier)
        return get_ip(carrier) + self._extra_ips[carrier]

    def add_ip(self, carrier, ip):
        carrier = normalize_carrier_name(carrier)
        self._extra_ips[carrier].append(IP(ip))



# -*- coding: utf-8 -*-
import urllib2
from lxml import etree

class CIDR(object):
    # subclass must override this property
    url = None

    def scrape(self):
        stream = self.get_stream()
        doc = etree.parse(stream, etree.HTMLParser())
        res = []
        for addr in self.do_scrape(doc):
            res.append(str(addr))
        return res

    def get_stream(self):
        return urllib2.urlopen(self.url)

    def do_scrape(self, doc):
        raise NotImplementedError()


class DoCoMoCIDR(CIDR):
    url = 'http://www.nttdocomo.co.jp/service/imode/make/content/ip/'

    def do_scrape(self, doc):
        return [x.text for x in doc.xpath('//div[@class="boxArea" and count(preceding-sibling::*)=2]/div/div[@class="section"]/ul[@class="normal txt" and position()=1]/li')]


class EZWebCIDR(CIDR):
    url = 'http://www.au.kddi.com/ezfactory/tec/spec/ezsava_ip.html'

    def do_scrape(self, doc):
        res = []
        rows = doc.xpath("""//table[@cellspacing="1"]/tr[@bgcolor="#ffffff"]""")
        for row in rows:
            cols = row.xpath('./td/div[@class="TableText"]/text()')
            if len(cols) == 4:
                # deprecated
                continue
            res.append('%s%s' % (cols[1], cols[2]))
        return res


class SoftBankCIDR(CIDR):
    url = 'http://creation.mb.softbank.jp/web/web_ip.html'

    def do_scrape(self, doc):
        return [x.text.strip() for x in doc.xpath("//div[@class='contents']/table/tr[7]/td/table/tr/td/table/tr/td")]


class WILLCOMCIDR(CIDR):
    url = 'http://www.willcom-inc.com/ja/service/contents_service/create/center_info/index.html'

    def do_scrape(self, doc):
        res = []
        sep = 0
        for td in doc.xpath("//table[@width='100%' and @cellspacing='1' and @cellpadding='3']/tr/td"):
            if td.attrib.get('colspan') == "4":
                sep += 1
                if sep > 1:
                    break
            else:
                if td.attrib.get('align') == 'center' and td.attrib.get('bgcolor') == 'white':
                    txt = td[0].text
                    if txt:
                        res.append(txt)
        return res


def scrape_cidr(carrier):
    carrier = carrier.lower()

    alias = { 'kddi'      : 'ezweb',
              'thirdforce': 'softbank',
              }.get(carrier)
    if alias is not None:
        carrier = alias

    try:
        s = { 'docomo'  : DoCoMoCIDR(),
              'ezweb'   : EZWebCIDR(),
              'softbank': SoftBankCIDR(),
              'willcom' : WILLCOMCIDR(),
              }[carrier]
        return s.scrape()
    except KeyError:
        raise ValueError('invalid carrier name "%s"' % carrier)



# -*- coding: utf-8 -*-

class AbstractUserAgentFactory(object):
    parser = None
    device_class = None

    def create(self, environ, context):
        params = self.parser.parse(environ.get('HTTP_USER_AGENT', ''))
        device = self.device_class(environ, context)
        for k, v in params.items():
            setattr(device, k, v)
        return device



# -*- coding: utf-8 -*-
from uamobile.base import UserAgent, Display

class SoftBankUserAgent(UserAgent):
    carrier = 'SoftBank'
    short_carrier = 'S'
    serialnumber = None

    def supports_cookie(self):
        """
        returns True if the device supports HTTP Cookie.
        For more information, see
        http://www2.developers.softbankmobile.co.jp/dp/tool_dl/download.php?docid=119
        """
        return self.is_3g() or self.is_type_w()

    def strip_serialnumber(self):
        """
        strip Device ID(Hardware ID)
        """
        if not self.serialnumber:
            return super(SoftBankUserAgent, self).strip_serialnumber()

        return self.useragent.replace('/SN%s' % self.serialnumber, '')

    def is_softbank(self):
        return True

    def is_vodafone(self):
        return True

    def is_jphone(self):
        return True

    def is_3g(self):
        return self._is_3g

    def is_type_c(self):
        """
        returns True if the type is C.
        """
        return not self._is_3g and (self.version.startswith('3.') or self.version.startswith('2.'))

    def is_type_p(self):
        """
        returns True if the type is P.
        """
        return not self._is_3g and self.version.startswith('4.')

    def is_type_w(self):
        """
        returns True if the type is W.
        """
        return not self._is_3g and self.version.startswith('5.')

    def get_jphone_uid(self):
        """
        returns the x-jphone-uid
        for the information about x-jphone-uid, see
        http://developers.softbankmobile.co.jp/dp/tool_dl/web/tech.php
        """
        try:
            return self.environ['HTTP_X_JPHONE_UID']
        except KeyError:
            return None
    jphone_uid = property(get_jphone_uid)

    def get_msname(self):
        return self.environ.get('HTTP_X_JPHONE_MSNAME')
    msname = property(get_msname)

    def get_smaf(self):
        return self.environ.get('HTTP_X_JPHONE_SMAF')
    smaf = property(get_smaf)

    def make_display(self):
        """
        create a new Display object.
        """
        try:
            width, height = map(int, self.environ.get('HTTP_X_JPHONE_DISPLAY', '').split('*', 1))
        except ValueError:
            # x-jphone-display is absent, or invalid format
            width = None
            height = None

        # assuming that WSGI environment variable is always string if the key exists
        color_string = self.environ.get('HTTP_X_JPHONE_COLOR', '')
        color = color_string.startswith('C')

        try:
            depth = int(color_string[1:])
        except (ValueError, TypeError):
            depth = 0

        return Display(width=width, height=height, color=color, depth=depth)



# -*- coding: utf-8 -*-
from uamobile.factory.base import AbstractUserAgentFactory
from uamobile.softbank import SoftBankUserAgent
from uamobile.parser import SoftBankUserAgentParser

class SoftBankUserAgentFactory(AbstractUserAgentFactory):
    device_class = SoftBankUserAgent
    parser       = SoftBankUserAgentParser()




# -*- coding: utf-8 -*-
from uamobile.factory.base import AbstractUserAgentFactory
from uamobile.willcom import WillcomUserAgent
from uamobile.parser import WillcomUserAgentParser

class WillcomUserAgentFactory(AbstractUserAgentFactory):
    device_class = WillcomUserAgent
    parser       = WillcomUserAgentParser()




# -*- coding: utf-8 -*-
import re
from uamobile.nonmobile import NonMobileUserAgent as NonMobile
from uamobile.context import Context

__version__ = '0.2.6'
__author__  = 'Chihiro Sakatoku <csakatoku@gmail.com>'

__all__ = ['detect_fast', 'detect', 'Context', 'NonMobile']

DOCOMO_RE   = re.compile(r'^DoCoMo/\d\.\d[ /]')
SOFTBANK_RE = re.compile(r'^(?:(?:SoftBank|Vodafone|J-PHONE)/\d\.\d|MOT-)')
EZWEB_RE    = re.compile(r'^(?:KDDI-[A-Z]+\d+[A-Z]? )?UP\.Browser\/')
WILLCOM_RE  = re.compile(r'^Mozilla/3\.0\((?:DDIPOCKET|WILLCOM);|^Mozilla/4\.0 \(compatible; MSIE (?:6\.0|4\.01); Windows CE; SHARP/WS\d+SH; PPC; \d+x\d+\)')

def detect_fast(useragent):
    """
    return name of Japanese mobile carriers from a given useragent string.
    if the agent isn't mobile one, then returns 'nonmobile'.
    """
    if DOCOMO_RE.match(useragent):
        return "docomo"
    elif EZWEB_RE.match(useragent):
        return "ezweb"
    elif SOFTBANK_RE.match(useragent):
        return "softbank"
    elif WILLCOM_RE.match(useragent):
        return "willcom"
    else:
        return "nonmobile"

def detect(environ, context=None):
    """
    parse HTTP user agent string and detect a mobile device.
    """
    context = context or Context()
    try:
        ## if key 'HTTP_USER_AGENT' doesn't exist,
        ## we are not able to decide agent class in the first place.
        ## so raise KeyError to return NonMobile agent.
        carrier = detect_fast(environ['HTTP_USER_AGENT'])

        ## if carrier is 'nonmobile', raise KeyError intentionally
        factory_class = {
            'docomo'  : context.docomo_factory,
            'ezweb'   : context.ezweb_factory,
            'softbank': context.softbank_factory,
            'willcom' : context.willcom_factory,
            }[carrier]
        return factory_class().create(environ, context)
    except KeyError:
        return NonMobile(environ, context)



# -*- coding: utf-8 -*-
from uamobile.base import UserAgent, Display

class WillcomUserAgent(UserAgent):
    name = 'WILLCOM'
    carrier = 'WILLCOM'
    short_carrier = 'W'

    def __init__(self, *args, **kwds):
        super(WillcomUserAgent, self).__init__(*args, **kwds)
        self.serialnumber = None
        self.model_version = None
        self.browser_version = None

    def supports_cookie(self):
        # TODO
        # I'm not sure All WILLCOM phones can handle HTTP cookie.
        return True

    def make_display(self):
        """
        create a new Display object.
        """
        return Display()

    def is_willcom(self):
        return True

    def is_airhphone(self):
        return True



# -*- coding: utf-8 -*-
from uamobile import cidr

class UserAgent(object):
    """
    Base class representing HTTTP user agent.
    """

    def __init__(self, environ, context):
        try:
            self.useragent = environ['HTTP_USER_AGENT']
        except KeyError, e:
            self.useragent = ''
        self.environ = environ
        self.context = context

        self.model = ''
        self.version = ''
        self._display = None

    def __repr__(self):
        return '<%s "%s">' % (self.__class__.__name__, self.useragent)

    def __str__(self):
        return self.useragent

    def strip_serialnumber(self):
        """
        strip serialnumber such as FOMA card ID of docomo
        adn return a normalized User-Agent string.
        """
        return self.useragent

    def get_display(self):
        """
        returns Display object.
        """
        if self._display is None:
            self._display = self.make_display()
        return self._display
    display = property(get_display)

    def make_display(self):
        raise NotImplementedError

    def is_docomo(self):
        """
        returns True if the agent is DoCoMo.
        """
        return False

    def is_ezweb(self):
        """
        returns True if the agent is EZweb.
        """
        return False

    def is_tuka(self):
        """
        returns True if the agent is TU-Ka.
        """
        return False

    def is_softbank(self):
        """
        returns True if the agent is Softbank.
        """
        return False

    def is_vodafone(self):
        """
        returns True if the agent is Vodafone (now SotBank).
        """
        return False

    def is_jphone(self):
        """
        returns True if the agent is J-PHONE (now softbank).
        """
        return False

    def is_willcom(self):
        """
        returns True if the agent is Willcom.
        """
        return False

    def is_airhphone(self):
        """
        returns True if the agent is AirH'PHONE.
        """
        return False

    def is_wap1(self):
        return False

    def is_wap2(self):
        return False

    def is_nonmobile(self):
        return False

    def supports_cookie(self):
        """
        returns True if the agent supports HTTP cookie.
        """
        raise NotImplementedError

    def _get_real_ip(self):
        """
        get real IP address of client from REMOTE_ADDR and
        HTTP_X_FORWARDED_FOR variables
        """
        try:
            remote_addr = self.environ['REMOTE_ADDR']
        except KeyError:
            return None

        forwared_for = self.environ.get('HTTP_X_FORWARDED_FOR')
        proxy_ips = self.context.get_proxy_ips()
        if forwared_for and proxy_ips:
            forwared_for = forwared_for.split(',', 1)[0].strip()

            # check REMOTE_ADDR is included in trusted reverse
            # proxy address
            for addr in proxy_ips:
                if remote_addr in addr:
                    # override remote addr
                    remote_addr = forwared_for
                    break

        try:
            return cidr.IP(remote_addr)
        except ValueError:
            # invalid ip address, like 'unknown'
            return None

    def get_real_ip(self):
        try:
            return self._real_ip
        except AttributeError:
            self._real_ip = self._get_real_ip()
            return self._real_ip

    def is_crawler(self):
        """
        return True if the client is a mobile crawler
        """
        remote_addr = self.get_real_ip()
        if remote_addr is None:
            # who knows?
            return False

        for addr in self.context.get_ip('crawler'):
            if remote_addr in addr:
                return True

        return False

    def is_bogus(self):
        """
        return True if the client isn't accessing via gateways of
        japanese mobile carrier
        """
        remote_addr = self.get_real_ip()
        if remote_addr is None:
            return True

        for addr in self.context.get_ip(self.carrier):
            if remote_addr in addr:
                return False

        return True


class Display(object):
    """
    Display information for mobile devices.
    """

    def __init__(self, width=240, height=320, depth=262144, color=1,
                 width_bytes=None, height_bytes=None):
        self.width  = width
        self.height = height
        self.depth  = depth
        self.color  = color
        self.width_bytes = width_bytes
        self.height_bytes = height_bytes

    def is_qvga(self):
        return self.width >= 240

    def is_vga(self):
        return self.width >= 480



# -*- coding: utf-8 -*-
from pkg_resources import resource_string
from IPy import IP

from uamobile.cidrdata import crawler, docomo, ezweb, softbank, willcom

__all__ = ['IP', 'get_ip_addrs', 'get_ip']

def get_ip_addrs(carrier):
    carrier = carrier.lower()
    if carrier not in ('docomo', 'ezweb', 'softbank', 'willcom', 'crawler', 'nonmobile'):
        raise ValueError('invalid carrier name "%s"' % carrier)

    return { 'docomo'   : docomo.DATA,
             'ezweb'    : ezweb.DATA,
             'softbank' : softbank.DATA,
             'willcom'  : willcom.DATA,
             'crawler'  : crawler.DATA,
             'nonmobile': [ '0.0.0.0/0' ],
             }[carrier]

def get_ip(carrier, _memo={}):
    try:
        return _memo[carrier]
    except KeyError:
        _memo[carrier] = [IP(x) for x in get_ip_addrs(carrier)]
        return _memo[carrier]




# -*- coding: utf-8 -*-
import re

class UserAgentParser(object):
    def parse(self, useragent):
        raise NotImplementedError()

###################################
# docomo
###################################

DOCOMO_STATUS_SET = ('TB', 'TC', 'TD', 'TJ')

DOCOMO_VENDOR_RE = re.compile(r'([A-Z]+)\d')

FOMA_SERIES_4DIGITS_RE = re.compile(r'\d{4}')
FOMA_SERIES_3DIGITS_RE = re.compile(r'(\d{3}i|\d{2}[ABC][23]?)')

DOCOMO_COMMENT_RE = re.compile(r'\((.+?)\)')
DOCOMO_DISPLAY_BYTES_RE = re.compile(r'^W(\d+)H(\d+)$')

DOCOMO_HTML_VERSION_LIST = [
        (re.compile('[DFNP]501i'), '1.0'),
        (re.compile('502i|821i|209i|651|691i|(?:F|N|P|KO)210i|^F671i$'), '2.0'),
        (re.compile('(?:D210i|SO210i)|503i|211i|SH251i|692i|200[12]|2101V'), '3.0'),
        (re.compile('504i|251i|^F671iS$|212i|2051|2102V|661i|2701|672i|SO213i|850i'), '4.0'),
        (re.compile('eggy|P751v'), '3.2'),
        (re.compile('505i|252i|900i|506i|880i|253i|P213i|901i|700i|851i$|701i|881i|^SA800i$|600i|^L601i$|^M702i(?:S|G)$|^L602i$'), '5.0'),
        (re.compile('902i|702i|851iWM|882i|883i$|^N601i$|^D800iDS$|^P70[34]imyu$'), '6.0'),
        (re.compile('883iES|903i|703i|904i|704i'), '7.0'),
        (re.compile('905i|705i'), '7.1'),
        (re.compile('906i|[01][0-9]A[23]?'), '7.2'),
        ]

def _get_docomo_html_version(model):
    """
    returns supported HTML version like '3.0'.
    if unkwon, return None.
    """
    for pattern, value in DOCOMO_HTML_VERSION_LIST:
        if pattern.search(model):
            return value
    return None

class DoCoMoUserAgentParser(UserAgentParser):
    def parse(self, useragent):
        if not useragent.startswith('DoCoMo/'):
            # this useragent isn't one of docomo in the first place
            return {}

        main, foma_or_comment = (useragent.split(' ', 1) + [None])[:2]

        if not foma_or_comment:
            # DoCoMo/1.0/R692i/c10
            res = self._parse_main(main)
        else:
            if foma_or_comment[0] == '(':
                # DoCoMo/1.0/P209is (Google CHTML Proxy/1.0)
                res = self._parse_main(main)
                res['comment'] = foma_or_comment[1:-1]
            else:
                # DoCoMo/2.0 N2001(c10;ser0123456789abcde;icc01234567890123456789)
                res = self._parse_foma(foma_or_comment)
                _, version = main.split('/')
                res['version'] = version

        model = res.get('model', '')

        # get vender code
        matcher = DOCOMO_VENDOR_RE.match(model)
        if matcher:
            res['vendor'] = matcher.group(1)

        return res

    def _parse_main(self, main):
        """
        parse main part of HTTP_USER_AGENT string (not foma)
        """
        _, version, model, cache, rest = (main.split('/', 4) + [None, None, None, None])[:5]
        if model == 'SH505i2':
            model = 'SH505i'

        # Get series
        if model == 'P651ps':
            series = '651'
        else:
            matcher = FOMA_SERIES_3DIGITS_RE.search(model)
            if matcher:
                series = matcher.group(1)
            else:
                series = None

        params = { 'model': model,
                   'version': version,
                   'status' : None,
                   'ser'    : None,
                   's'      : None,
                   'display_bytes': None,
                   'series' : series,
                   'html_version' : _get_docomo_html_version(model),
                   }
        if cache:
            try:
                params['c'] = int(cache[1:])
            except ValueError:
                pass

        if rest:
            for value in rest.split('/'):
                if value in DOCOMO_STATUS_SET:
                    params['status'] = value
                    continue

                if value.startswith('ser') and len(value) == 14:
                    params['ser'] = value[3:]
                    continue

                if value.startswith('s'):
                    try:
                        params['s'] = int(value[1:])
                        continue
                    except ValueError:
                        pass

                matcher = DOCOMO_DISPLAY_BYTES_RE.match(value)
                if matcher:
                    params['display_bytes'] = (int(matcher.group(1)), int(matcher.group(2)))
                    continue

        return params

    def _parse_foma(self, foma):
        # remove comment
        matcher = DOCOMO_COMMENT_RE.search(foma)
        if matcher:
            # use only first comment and ignore the rest
            # such as,
            # "DoCoMo/2.0 P900i(c100;TB;W24H11)(compatible; ichiro/mobile goo; +http://help.goo.ne.jp/door/crawler.html)"
            foma_params = matcher.group(1)
        else:
            foma_params = ''

        model = foma.split('(', 1)[0]

        if model == 'MST_v_SH2101V':
            model = 'SH2101V'

        # Get FOMA series number
        if FOMA_SERIES_4DIGITS_RE.search(model):
            ## agents such as "DoCoMo/2.0 P2102V(c100;TB)"
            series = 'FOMA'
        else:
            matcher = FOMA_SERIES_3DIGITS_RE.search(model)
            if matcher:
                series = matcher.group(1)
            else:
                series = None

        params = { 'model'        : model,
                   'status'       : None,
                   'ser'          : None,
                   'icc'          : None,
                   'display_bytes': None,
                   'series'       : series,
                   'html_version' : _get_docomo_html_version(model),
                   }
        for value in foma_params.split(';'):
            if value in DOCOMO_STATUS_SET:
                params['status'] = value
                continue

            if value.startswith('ser') and len(value) == 18:
                params['ser'] = value[3:]
                continue

            if value.startswith('icc') and len(value) == 23:
                params['icc'] = value[3:]
                continue

            if value.startswith('c'):
                try:
                    params['c'] = int(value[1:])
                    continue
                except ValueError:
                    pass

            matcher = DOCOMO_DISPLAY_BYTES_RE.match(value)
            if matcher:
                params['display_bytes'] = (int(matcher.group(1)), int(matcher.group(2)))
                continue

        return params


###################################
# EZWeb
###################################

UP_BROWSER_COMMENT_RE = re.compile('^\((.*)\)$')

class EZwebUserAgentParser(UserAgentParser):
    def parse(self, useragent):
        if useragent.startswith('KDDI-'):
            return self._parse_kddi(useragent)
        else:
            return self._parse_upbrowser(useragent)

    def _parse_kddi(self, useragent):
        """
        Parse User-Agent like,
        - KDDI-TS21 UP.Browser/6.0.2.276 (GUI) MMP/1.1
        - KDDI-TS3A UP.Browser/6.2.0.11.2.1 (GUI) MMP/2.0, Mozilla/4.08 (MobilePhone; NMCS/3.3) NetFront/3.3
        """
        # Remove "KDDI-"
        useragent = useragent[5:]

        name = None
        version = None
        device_id, browser, opt, server = useragent.split(' ', 4)[:4]
        if server.endswith(','):
            server = server[:-1]

        name, version = browser.split('/')
        version = '%s %s' % (version, opt)

        return { 'xhtml_compliant': True,
                 'device_id'      : device_id,
                 'model'          : device_id,
                 'server'         : server,
                 'version'        : version,
                 'name'           : name,
                 'comment'        : None,
                 }

    def _parse_upbrowser(self, useragent):
        """
        Parse User-Agent like,
        - UP.Browser/3.01-HI01 UP.Link/3.4.5.2
        """
        browser, server, comment = (useragent.split(' ', 2) + [None, None])[:3]
        name, software = browser.split('/', 1)
        version, device_id = software.split('-', 1)
        if comment:
            comment = UP_BROWSER_COMMENT_RE.sub(lambda m: m.group(1), comment)

        return { 'xhtml_compliant': False,
                 'device_id'      : device_id,
                 'model'          : device_id,
                 'server'         : server,
                 'version'        : version,
                 'name'           : name,
                 'comment'        : comment,
                 }

###################################
# SoftBank
###################################

VODAFONE_VENDOR_RE = re.compile(r'V\d+([A-Z]+)')
JPHONE_VENDOR_RE = re.compile(r'J-([A-Z]+)')
SOFTBANK_CRAWLER_RE = re.compile(r'\([^)]+\)')

class SoftBankUserAgentParser(UserAgentParser):

    def parse(self, useragent):
        """
        parse the useragent which starts with "SoftBank", "Vodafone", "J-PHONE", or "MOT-".
        """
        # strip crawler infomation such as,
        # J-PHONE/2.0/J-SH03 (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)
        # Vodafone/1.0/V705SH (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)
        # SoftBank/1.0/913SH/SHJ001/SN000123456789000 Browser/NetFront/3.4 Profile/MIDP-2.0 (symphonybot1.froute.jp; +http://search.froute.jp/howto/crawler.html)
        ua = SOFTBANK_CRAWLER_RE.sub('', useragent)
        ua = ua.strip().split(' ')

        carrier = ua[0]
        if carrier.startswith('SoftBank') or carrier.startswith('Vodafone'):
            return self._parse_vodaphone(ua)
        elif carrier.startswith('J-PHONE'):
            return self._parse_jphone(ua)
        elif carrier.startswith('MOT-'):
            return self._parse_motorola(ua)
        else:
            return {}

    def _parse_vodaphone(self, ua):
        # Vodafone/1.0/V702NK/NKJ001 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V702NK/NKJ001/SN123456789012345 Series60/2.6 Nokia6630/2.39.148 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V802SE/SEJ001/SN123456789012345 Browser/SEMC-Browser/4.1 Profile/MIDP-2.0 Configuration/CLDC-1.1
        # Vodafone/1.0/V705SH (compatible; Y!J-SRD/1.0; http://help.yahoo.co.jp/help/jp/search/indexing/indexing-27.html)

        (name,
         version,
         model,
         vendor_info,
         serial) = (ua[0].split('/') + [None, None, None, None])[:5]

        serialnumber = None
        if serial and serial.startswith('SN'):
            serialnumber = serial[2:]

        if not vendor_info:
            vendor = None
            vendor_version = None
        else:
            vendor = vendor_info[:-4]
            vendor_version = vendor_info[-4:]

        java_info = dict([x.split('/') for x in ua[2:] if x])

        return { 'packet_compliant': True,
                 'version'         : version,
                 'model'           : model,
                 'vendor'          : vendor,
                 'vendor_version'  : vendor_version,
                 'serialnumber'    : serialnumber,
                 'java_info'       : java_info,
                 '_is_3g'          : True,
                 }

    def _parse_jphone(self, ua):
        serialnumber = None
        vendor = None
        vendor_version = None
        java_info = {}

        if len(ua) > 1:
            # J-PHONE/4.0/J-SH51/SNJSHA3029293 SH/0001aa Profile/MIDP-1.0 Configuration/CLDC-1.0 Ext-Profile/JSCL-1.1.0
            packet_compliant = True

            (name,
             version,
             model,
             serial) = (ua[0].split('/') + [None, None, None])[:4]
            if serial and serial.startswith('SN'):
                serialnumber = serial[2:]

            try:
                vendor, vendor_version = ua[1].split('/')
            except ValueError, e:
                pass

            java_info.update([x.split('/') for x in ua[2:]])
        else:
            # J-PHONE/2.0/J-DN02
            packet_compliant = False

            name, version, model = ua[0].split('/')
            if model:
                matcher = VODAFONE_VENDOR_RE.match(model)
                if matcher:
                    vendor = matcher.group(1)
                else:
                    matcher = JPHONE_VENDOR_RE.match(model)
                    if matcher:
                        vendor = matcher.group(1)

        return { 'packet_compliant': packet_compliant,
                 'version'         : version,
                 'model'           : model,
                 'vendor'          : vendor,
                 'vendor_version'  : vendor_version,
                 'serialnumber'    : serialnumber,
                 'java_info'       : java_info,
                 '_is_3g'          : False,
                 }

    def _parse_motorola(self, ua):
        """
        parse HTTP_USER_AGENT string for the Motorola 3G agent
        """
        # MOT-V980/80.2F.2E. MIB/2.2.1 Profile/MIDP-2.0 Configuration/CLDC-1.1
        name, vendor_version = ua[0].split('/')
        model = name[name.rindex('-')+1:]

        java_info = dict([x.split('/') for x in ua[2:]])

        return { 'packet_compliant': True,
                 'vendor'          : 'MOT',
                 'vendor_version'  : vendor_version,
                 'model'           : model,
                 'java_info'       : java_info,
                 '_is_3g'          : True,
                 }



###################################
# WILLCOM
###################################

WILLCOM_RE = re.compile(r'^Mozilla/3\.0\((?:DDIPOCKET|WILLCOM);(.*)\)')
WILLCOM_CACHE_RE = re.compile(r'^[Cc](\d+)')
WINDOWS_CE_RE = re.compile(r'^Mozilla/4\.0 \((.*)\)')

class WillcomUserAgentParser(UserAgentParser):
    def parse(self, useragent):
        if useragent.startswith('Mozilla/4.0'):
            return self._parse_windows_ce(useragent)

        matcher = WILLCOM_RE.match(useragent)
        if not matcher:
            return {}

        (vendor,
         model,
         model_version,
         browser_version,
         cache) = matcher.group(1).split('/')

        cache_size = None
        if cache and cache.lower().startswith('c'):
            try:
                cache_size = int(cache[1:])
            except ValueError:
                pass

        return { 'vendor'         : vendor,
                 'model'          : model,
                 'model_version'  : model_version,
                 'browser_version': browser_version,
                 'cache_size'     : cache_size,
                 }

    def _parse_windows_ce(self, useragent):
        # Mozilla/4.0 (compatible; MSIE 4.01; Windows CE; SHARP/WS007SH; PPC; 480x640)
        matcher = WINDOWS_CE_RE.match(useragent)
        if not matcher:
            return {}

        try:
            _, browser_version, os, vendor_and_model, arch, width_and_height = matcher.group(1).split(';')
            vendor, model = vendor_and_model.strip().split('/')

            return { 'vendor'         : vendor.strip(),
                     'model'          : model.strip(),
                     'browser_version': browser_version.strip(),
                     'cache_size'     : None,
                     'os'             : os.strip(),
                     'arch'           : arch.strip(),
                     'display_bytes'  : width_and_height.strip(),
                     }
        except ValueError, e:
            return {}





# -*- coding: utf-8 -*-
from uamobile.factory.base import AbstractUserAgentFactory
from uamobile.docomo import DoCoMoUserAgent
from uamobile.parser import DoCoMoUserAgentParser

class DoCoMoUserAgentFactory(AbstractUserAgentFactory):
    device_class = DoCoMoUserAgent
    parser       = DoCoMoUserAgentParser()




from setuptools import setup, find_packages
import sys, os

version = '0.2.7'

setup(name='uamobile',
      version=version,
      description="WSGIUserAgentMobile - mobile user agent string parser for WSGI applications",
      long_description="""\
WSGIUserAgentMobile is HTTP mobile user agent string parser. It'll be useful in parsing HTTP_USER_AGENT strings of (mainly Japanese) mobile devices.

This library is ported from similar libraries in Perl and PHP and owes a lot to them.

HTTP-MobileAgent? http://search.cpan.org/~kurihara/HTTP-MobileAgent-0.26/lib/HTTP/MobileAgent.pm

PEAR::Net_UserAgent_Mobile? http://pear.php.net/package/Net_UserAgent_Mobile
""",
      classifiers=filter(None, map(str.strip, """\
Development Status :: 4 - Beta
License :: OSI Approved :: MIT License
Programming Language :: Python
Operating System :: OS Independent
Topic :: Software Development :: Libraries :: Python Modules
Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware
""".splitlines())),
      keywords='',
      author='Chihio Sakatoku',
      author_email='csakatoku@gmail.com',
      url='http://code.google.com/p/wsgiuseragentmobile/',
      license='MIT License',
      platforms=["any"],
      packages=find_packages(exclude=['examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
        'IPy',
      ],
      entry_points="""
      # -*- Entry points: -*-
      """,
      test_suite='nose.collector'
      )



# -*- coding: utf-8 -*-
from uamobile.base import UserAgent, Display

class EZwebUserAgent(UserAgent):
    carrier = 'EZweb'
    short_carrier = 'E'

    def supports_cookie(self):
        return True

    def make_display(self):
        """
        create a Display object.
        """
        env = self.environ
        try:
            width, height = map(int, env['HTTP_X_UP_DEVCAP_SCREENPIXELS'].split(',', 1))
        except (KeyError, ValueError), e:
            width = None
            height = None

        try:
            color = env['HTTP_X_UP_DEVCAP_ISCOLOR'] == '1'
        except KeyError:
            color = False

        try:
            sd = env['HTTP_X_UP_DEVCAP_SCREENDEPTH'].split(',', 1)
            depth = sd[0] and (2 ** int(sd[0])) or 0
        except (KeyError, ValueError):
            depth = None

        return Display(width=width, height=height, color=color, depth=depth)

    def is_ezweb(self):
        return True

    def get_serialnumber(self):
        """
        return the EZweb subscriber ID.
        if no subscriber ID is available, returns None.
        """
        try:
            return self.environ['HTTP_X_UP_SUBNO']
        except KeyError:
            return None
    serialnumber = property(get_serialnumber)

    def is_xhtml_compliant(self):
        """
        returns whether it's XHTML compliant or not.
        """
        return self.xhtml_compliant

    def is_win(self):
        """
        returns whether the agent is CDMA 1X WIN or not.
        """
        return self.device_id[2:3] == '3'

    def is_wap1(self):
        return not self.is_wap2()

    def is_wap2(self):
        return self.xhtml_compliant
